var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
};
import { PublicKey, SystemProgram, SYSVAR_CLOCK_PUBKEY, SYSVAR_RENT_PUBKEY, } from "@solana/web3.js";
import { isBytes, parseGovernanceVaa, } from "../../../vaa";
import { createReadOnlyWormholeProgramInterface } from "../program";
import { deriveWormholeBridgeDataKey, deriveClaimKey, deriveFeeCollectorKey, deriveGuardianSetKey, derivePostedVaaKey, deriveUpgradeAuthorityKey, } from "../accounts";
import { BpfLoaderUpgradeable, deriveUpgradeableProgramKey } from "../../utils";
export function createSetFeesInstruction(wormholeProgramId, payer, vaa) {
    var methods = createReadOnlyWormholeProgramInterface(wormholeProgramId).methods.setFees();
    // @ts-ignore
    return methods._ixFn.apply(methods, __spreadArray(__spreadArray([], __read(methods._args)), [{
            accounts: getSetFeesAccounts(wormholeProgramId, payer, vaa),
            signers: undefined,
            remainingAccounts: undefined,
            preInstructions: undefined,
            postInstructions: undefined,
        }]));
}
export function getSetFeesAccounts(wormholeProgramId, payer, vaa) {
    var parsed = isBytes(vaa) ? parseGovernanceVaa(vaa) : vaa;
    return {
        payer: new PublicKey(payer),
        bridge: deriveWormholeBridgeDataKey(wormholeProgramId),
        vaa: derivePostedVaaKey(wormholeProgramId, parsed.hash),
        claim: deriveClaimKey(wormholeProgramId, parsed.emitterAddress, parsed.emitterChain, parsed.sequence),
        systemProgram: SystemProgram.programId,
    };
}
export function createTransferFeesInstruction(wormholeProgramId, payer, recipient, vaa) {
    var methods = createReadOnlyWormholeProgramInterface(wormholeProgramId).methods.transferFees();
    // @ts-ignore
    return methods._ixFn.apply(methods, __spreadArray(__spreadArray([], __read(methods._args)), [{
            accounts: getTransferFeesAccounts(wormholeProgramId, payer, recipient, vaa),
            signers: undefined,
            remainingAccounts: undefined,
            preInstructions: undefined,
            postInstructions: undefined,
        }]));
}
export function getTransferFeesAccounts(wormholeProgramId, payer, recipient, vaa) {
    var parsed = isBytes(vaa) ? parseGovernanceVaa(vaa) : vaa;
    return {
        payer: new PublicKey(payer),
        bridge: deriveWormholeBridgeDataKey(wormholeProgramId),
        vaa: derivePostedVaaKey(wormholeProgramId, parsed.hash),
        claim: deriveClaimKey(wormholeProgramId, parsed.emitterAddress, parsed.emitterChain, parsed.sequence),
        feeCollector: deriveFeeCollectorKey(wormholeProgramId),
        recipient: new PublicKey(recipient),
        rent: SYSVAR_RENT_PUBKEY,
        systemProgram: SystemProgram.programId,
    };
}
export function createUpgradeGuardianSetInstruction(wormholeProgramId, payer, vaa) {
    var methods = createReadOnlyWormholeProgramInterface(wormholeProgramId).methods.upgradeGuardianSet();
    // @ts-ignore
    return methods._ixFn.apply(methods, __spreadArray(__spreadArray([], __read(methods._args)), [{
            accounts: getUpgradeGuardianSetAccounts(wormholeProgramId, payer, vaa),
            signers: undefined,
            remainingAccounts: undefined,
            preInstructions: undefined,
            postInstructions: undefined,
        }]));
}
export function getUpgradeGuardianSetAccounts(wormholeProgramId, payer, vaa) {
    var parsed = isBytes(vaa) ? parseGovernanceVaa(vaa) : vaa;
    return {
        payer: new PublicKey(payer),
        bridge: deriveWormholeBridgeDataKey(wormholeProgramId),
        vaa: derivePostedVaaKey(wormholeProgramId, parsed.hash),
        claim: deriveClaimKey(wormholeProgramId, parsed.emitterAddress, parsed.emitterChain, parsed.sequence),
        guardianSetOld: deriveGuardianSetKey(wormholeProgramId, parsed.guardianSetIndex),
        guardianSetNew: deriveGuardianSetKey(wormholeProgramId, parsed.guardianSetIndex + 1),
        systemProgram: SystemProgram.programId,
    };
}
export function createUpgradeContractInstruction(wormholeProgramId, payer, vaa) {
    var methods = createReadOnlyWormholeProgramInterface(wormholeProgramId).methods.upgradeContract();
    // @ts-ignore
    return methods._ixFn.apply(methods, __spreadArray(__spreadArray([], __read(methods._args)), [{
            accounts: getUpgradeContractAccounts(wormholeProgramId, payer, vaa),
            signers: undefined,
            remainingAccounts: undefined,
            preInstructions: undefined,
            postInstructions: undefined,
        }]));
}
export function getUpgradeContractAccounts(wormholeProgramId, payer, vaa, spill) {
    var parsed = isBytes(vaa) ? parseGovernanceVaa(vaa) : vaa;
    var implementation = parsed.orderPayload;
    if (implementation.length != 32) {
        throw new Error("implementation.length != 32");
    }
    return {
        payer: new PublicKey(payer),
        bridge: deriveWormholeBridgeDataKey(wormholeProgramId),
        vaa: derivePostedVaaKey(wormholeProgramId, parsed.hash),
        claim: deriveClaimKey(wormholeProgramId, parsed.emitterAddress, parsed.emitterChain, parsed.sequence),
        upgradeAuthority: deriveUpgradeAuthorityKey(wormholeProgramId),
        spill: new PublicKey(spill === undefined ? payer : spill),
        implementation: new PublicKey(implementation),
        programData: deriveUpgradeableProgramKey(wormholeProgramId),
        wormholeProgram: new PublicKey(wormholeProgramId),
        rent: SYSVAR_RENT_PUBKEY,
        clock: SYSVAR_CLOCK_PUBKEY,
        bpfLoaderUpgradeable: BpfLoaderUpgradeable.programId,
        systemProgram: SystemProgram.programId,
    };
}
